using Autofac;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using System;
using IRaCIS.Core.Application.Filter;
using LogDashboard;
using MediatR;
using IRaCIS.Core.Application.MediatR.Handlers;
using Microsoft.Extensions.Logging;
using AspNetCoreRateLimit;
using Microsoft.AspNetCore.HttpOverrides;
using IRaCIS.Core.Infra.EFCore;
using System.Globalization;
using Microsoft.AspNetCore.Localization;
using Localization;
using Magicodes.ExporterAndImporter.Core.Filters;
using IRaCIS.Core.Application.MediatR.CommandAndQueries;
using IRaCIS.Core.Infra.EFCore.Common;
using Invio.Extensions.Authentication.JwtBearer;
using Microsoft.AspNetCore.SignalR;
using IRaCIS.Core.Domain.Share;
using Microsoft.AspNetCore.StaticFiles;
using IRaCIS.Application.Services.BackGroundJob;
using IRaCIS.Core.Application.Helper;
using Microsoft.AspNetCore.Http;
using Autofac.Core;
using DocumentFormat.OpenXml.InkML;
using EasyCaching.Core;

namespace IRaCIS.Core.API
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            _configuration = configuration;
        }
        public ILogger<Startup> _logger { get; }

        public IConfiguration _configuration { get; }

        //// ConfigureContainer is where you can register things directly
        //// with Autofac. This runs after ConfigureServices so the things
        //// here will override registrations made in ConfigureServices.
        //// Don't build the container; that gets done for you by the factory.
        // for castle
        public void ConfigureContainer(ContainerBuilder containerBuilder)
        {
            containerBuilder.RegisterModule<AutofacModuleSetup>();

            #region  Test
            //containerBuilder.RegisterType<ClinicalDataService>().PropertiesAutowired().InstancePerLifetimeScope();//עִ

            //var container = containerBuilder.Build();

            //// Now you can resolve services using Autofac. For example,
            //// this line will execute the lambda expression registered
            //// to the IConfigReader service.
            //using (var scope = container.BeginLifetimeScope())
            //{
            //    var reader = scope.Resolve<BaseService>();

            //    var test = scope.Resolve<ClinicalDataService>();
            //    var test2 = scope.Resolve<IClinicalDataService>();

            //    var test3 = scope.Resolve<IEFUnitOfWork<IRaCISDBContext>>();
            //}
            #endregion
        }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            //
            services.AddHealthChecks();
            //ػ
            services.AddJsonLocalization(options => options.ResourcesPath = "Resources");

            // 쳣ͳһ֤JsonлáַͳһTrim()
            services.AddControllers(options =>
            {
                //options.Filters.Add<LogActionFilter>();
                options.Filters.Add<ModelActionFilter>();
                options.Filters.Add<ProjectExceptionFilter>();
                options.Filters.Add<UnitOfWorkFilter>();

                if (_configuration.GetSection("BasicSystemConfig").GetValue<bool>("OpenLoginLimit"))
                {
                    options.Filters.Add<LimitUserRequestAuthorization>();
                }


            })
                .AddNewtonsoftJsonSetup(); // NewtonsoftJson л  

            services.AddOptions().Configure<SystemEmailSendConfig>(_configuration.GetSection("SystemEmailSendConfig"));
            services.AddOptions().Configure<ServiceVerifyConfigOption>(_configuration.GetSection("BasicSystemConfig"));
            services.AddOptions().Configure<AliyunOSSOptions>(_configuration.GetSection("AliyunOSS"));
            services.AddOptions().Configure<ObjectStoreServiceOptions>(_configuration.GetSection("ObjectStoreService"));


            //̬WebApi + UnifiedApiResultFilter  ʡ
            services.AddDynamicWebApiSetup();
            //AutoMapper
            services.AddAutoMapperSetup();
            //EF ORM  QueryWithNoLock
            services.AddEFSetup(_configuration);
            //Http Ӧѹ
            services.AddResponseCompressionSetup();
            //Swagger Api ĵ
            services.AddSwaggerSetup();
            //JWT Token ֤
            services.AddJWTAuthSetup(_configuration);
            // MediatR Ϣ ¼  ӳ עhandlerӦϵ
            services.AddMediatR(cfg => cfg.RegisterServicesFromAssemblyContaining<ConsistencyVerificationHandler>());
            // EasyCaching 
            services.AddEasyCachingSetup(_configuration);

            //services.AddDistributedMemoryCache();

            // hangfire  ʱ  н棬Ѻ~
            services.AddhangfireSetup(_configuration);

            //  QuartZ ʱ  ʹhangfire ʱãҪԴ򿪣Ѿ
            services.AddQuartZSetup(_configuration);

            // ϴļ
            //services.AddStaticFileAuthorizationSetup();


            ////HttpReports ʱ
            //services.AddHttpReports().AddHttpTransport();
            //Serilog ־ӻ LogDashboard־
            services.AddLogDashboardSetup();
            //ϴ 
            services.Configure<FormOptions>(options =>
            {
                options.MultipartBodyLengthLimit = int.MaxValue;
                options.ValueCountLimit = int.MaxValue;
                options.ValueLengthLimit = int.MaxValue;
            });
            //IP  ð ߺ
            //services.AddIpPolicyRateLimitSetup(_configuration);
            // û Ȩ
            //services.AddAuthorizationPolicySetup(_configuration);

            services.AddJsonConfigSetup(_configuration);
            //תͷ  ȡʵIP 
            services.Configure<ForwardedHeadersOptions>(options =>
            {
                options.ForwardedHeaders =
                    ForwardedHeaders.XForwardedFor | ForwardedHeaders.XForwardedProto;
            });
            //DicomӰȾͼƬ  ƽ̨
            services.AddDicomSetup();

            // ʵʱӦ
            services.AddSignalR();


            services.AddSingleton<IUserIdProvider, IRaCISUserIdProvider>();

            //services.AddSingleton<IImportResultFilter, ImportResultFilteTest>();

            services.AddMemoryCache();

        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public async void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            //app.UsePathBase(PathString.FromUriComponent("/api"));

            //ػ
            app.UseLocalization();

            app.UseForwardedHeaders();

            //Ӧѹ
            app.UseResponseCompression();

            //app.UseCors(t => t.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader());

            //Ҫ token ʵľ̬ļ  wwwroot css, JavaScript, and images don't require authentication.
            app.UseStaticFiles();


            //LogDashboard
            app.UseLogDashboard("/back/logs");

            //hangfire
            app.UseHangfireConfig(env);

            ////ʱ
            //app.UseHttpReports();

            //// м
            //app.UseIpRateLimiting();



            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                //app.UseHsts();
            }
            Console.WriteLine("ǰ " + env.EnvironmentName);

            //app.UseMiddleware<AuthMiddleware>();

            // 쳣  404 
            app.UseStatusCodePagesWithReExecute("/Error/{0}");

            SwaggerSetup.Configure(app, env);



            ////serilog ¼ûϢ
            app.UseSerilogConfig(env);

            app.UseRouting();

            app.UseCors(t => t.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader());

            //app.UseIRacisHostStaticFileStore(env);
            app.UseMiddleware<MultiDiskStaticFilesMiddleware>();


            app.UseAuthentication();
            //app.UseJwtBearerQueryString();
            app.UseAuthorization();

            ////ļŷ  Token 
            ////app.UseIRacisHostStaticFileStore(env);

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();

                endpoints.MapHub<UploadHub>("/UploadHub");

                endpoints.MapHealthChecks("/health");
            });


            var hangfireJobService = app.ApplicationServices.GetRequiredService<IIRaCISHangfireJob>();

            await hangfireJobService.InitHangfireJobTaskAsync();
            //еʱÿ
            //HangfireJobHelper.NotImmediatelyOnceOnlyJob<IIRaCISHangfireJob>(t => t.InitHangfireJobTaskAsync(),TimeSpan.FromSeconds(1));


        }
    }
}
